﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;

namespace HeuristicLab.BioBoost.Utils {
  public static class EnumerableExtensions {
    public static IEnumerable<Tuple<T1, T2>> Zip<T1, T2>(IEnumerable<T1> e1, IEnumerable<T2> e2) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext()) {
        yield return new Tuple<T1, T2>(it1.Current, it2.Current);
      }
    }
    public static IEnumerable<Tuple<T1, T2, T3>> Zip<T1, T2, T3>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext()) {
        yield return new Tuple<T1, T2, T3>(it1.Current, it2.Current, it3.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
    public static IEnumerable<Tuple<T1, T2, T3, T4>> Zip<T1, T2, T3, T4>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3, IEnumerable<T4> e4) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      var it4 = e4.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext() && it4.MoveNext()) {
        yield return new Tuple<T1, T2, T3, T4>(it1.Current, it2.Current, it3.Current, it4.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext() || it4.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
    public static IEnumerable<Tuple<T1, T2, T3, T4, T5>> Zip<T1, T2, T3, T4, T5>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3, IEnumerable<T4> e4, IEnumerable<T5> e5) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      var it4 = e4.GetEnumerator();
      var it5 = e5.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext() && it4.MoveNext() && it5.MoveNext()) {
        yield return new Tuple<T1, T2, T3, T4, T5>(it1.Current, it2.Current, it3.Current, it4.Current, it5.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext() || it4.MoveNext() || it5.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
    public static IEnumerable<Tuple<T1, T2, T3, T4, T5, T6>> Zip<T1, T2, T3, T4, T5, T6>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3, IEnumerable<T4> e4, IEnumerable<T5> e5, IEnumerable<T6> e6) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      var it4 = e4.GetEnumerator();
      var it5 = e5.GetEnumerator();
      var it6 = e6.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext() && it4.MoveNext() && it5.MoveNext() && it6.MoveNext()) {
        yield return new Tuple<T1, T2, T3, T4, T5, T6>(it1.Current, it2.Current, it3.Current, it4.Current, it5.Current, it6.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext() || it4.MoveNext() || it5.MoveNext() || it6.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
    public static IEnumerable<Tuple<T1, T2, T3, T4, T5, T6, T7>> Zip<T1, T2, T3, T4, T5, T6, T7>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3, IEnumerable<T4> e4, IEnumerable<T5> e5, IEnumerable<T6> e6, IEnumerable<T7> e7) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      var it4 = e4.GetEnumerator();
      var it5 = e5.GetEnumerator();
      var it6 = e6.GetEnumerator();
      var it7 = e7.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext() && it4.MoveNext() && it5.MoveNext() && it6.MoveNext() && it7.MoveNext()) {
        yield return new Tuple<T1, T2, T3, T4, T5, T6, T7>(it1.Current, it2.Current, it3.Current, it4.Current, it5.Current, it6.Current, it7.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext() || it4.MoveNext() || it5.MoveNext() || it6.MoveNext() || it7.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
    public static IEnumerable<Tuple<T1, T2, T3, T4, T5, T6, T7, T8>> Zip<T1, T2, T3, T4, T5, T6, T7, T8>(IEnumerable<T1> e1, IEnumerable<T2> e2, IEnumerable<T3> e3, IEnumerable<T4> e4, IEnumerable<T5> e5, IEnumerable<T6> e6, IEnumerable<T7> e7, IEnumerable<T8> e8) {
      var it1 = e1.GetEnumerator();
      var it2 = e2.GetEnumerator();
      var it3 = e3.GetEnumerator();
      var it4 = e4.GetEnumerator();
      var it5 = e5.GetEnumerator();
      var it6 = e6.GetEnumerator();
      var it7 = e7.GetEnumerator();
      var it8 = e8.GetEnumerator();
      while (it1.MoveNext() && it2.MoveNext() && it3.MoveNext() && it4.MoveNext() && it5.MoveNext() && it6.MoveNext() && it7.MoveNext() && it8.MoveNext()) {
        yield return new Tuple<T1, T2, T3, T4, T5, T6, T7, T8>(it1.Current, it2.Current, it3.Current, it4.Current, it5.Current, it6.Current, it7.Current, it8.Current);
      }
      if (it1.MoveNext() || it2.MoveNext() || it3.MoveNext() || it4.MoveNext() || it5.MoveNext() || it6.MoveNext() || it7.MoveNext() || it8.MoveNext())
        throw new InvalidOperationException("Iterators have different lengths");
    }
  }
}
